package com.nisum.example.mongodb.controller;

import com.nisum.example.mongodb.model.Employee;
import com.nisum.example.mongodb.repository.EmployeeRepository;
import com.nisum.example.mongodb.service.EmployeeServiceImpl;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.Mockito;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.reactive.WebFluxTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.context.annotation.Import;
import org.springframework.http.MediaType;
import org.springframework.test.context.junit.jupiter.SpringExtension;
import org.springframework.test.web.reactive.server.WebTestClient;
import org.springframework.web.reactive.function.BodyInserters;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

import java.util.Arrays;
import java.util.List;

import static org.mockito.Mockito.times;

@ExtendWith(SpringExtension.class)
@WebFluxTest(controllers = EmployeeController.class)
@Import(EmployeeServiceImpl.class)
public class EmployeeControllerTest {

    @MockBean
    private EmployeeRepository repository;

    @Autowired
    private WebTestClient webTestClient;

    @Test
    void testCreateEmployee(){
        Employee employee=new Employee();
        employee.setId(1);
        employee.setName("SAM");
        employee.setSalary(1500L);

        Mockito.when(repository.save(employee)).thenReturn(Mono.just(employee));

        webTestClient.post()
                .uri("/create")
                .contentType(MediaType.APPLICATION_JSON)
                .body(BodyInserters.fromValue(employee))
                .exchange()
                .expectStatus().isCreated();

        Mockito.verify(repository,times(1)).save(employee);
    }

    @Test
    void testGetEmployeeById(){
        Employee employee=new Employee();
        employee.setId(1);
        employee.setName("SAM");
        employee.setSalary(1500L);

        Mockito.when(repository.findById(1))
                .thenReturn(Mono.just(employee));

        webTestClient.get()
                .uri("/{id}",1)
                .exchange()
                .expectStatus().isOk()
                .expectBody()
                .jsonPath("$.name").isNotEmpty()
                .jsonPath("$.id").isEqualTo(1)
                .jsonPath("$.name").isEqualTo("SAM")
                .jsonPath("$.salary").isEqualTo(1500);

        Mockito.verify(repository,times(1))
                .findById(1);
    }

    @Test
    void testGetEmployeeByName(){
        Employee employee1=new Employee();
        employee1.setId(1);
        employee1.setName("SAM");
        employee1.setSalary(1500L);

        Employee employee2=new Employee();
        employee2.setId(2);
        employee2.setName("SAM");
        employee2.setSalary(2000L);

        List<Employee> employeeList= Arrays.asList(employee1,employee2);

        Flux<Employee> employeeFlux=Flux.fromIterable(employeeList);

        Mockito.when(repository.findByName("SAM"))
                .thenReturn(employeeFlux);

        webTestClient.get()
                .uri("/name/{name}","SAM")
                .exchange()
                .expectStatus().isOk()
                .expectBodyList(Employee.class)
                .hasSize(2);

        Mockito.verify(repository,times(1)).findByName("SAM");
    }

    @Test
    void testGetAllEmployees(){
        Employee employee1=new Employee();
        employee1.setId(1);
        employee1.setName("SAM");
        employee1.setSalary(1500L);

        Employee employee2=new Employee();
        employee2.setId(2);
        employee2.setName("JAMES");
        employee2.setSalary(2000L);

        List<Employee> employeeList=Arrays.asList(employee1,employee2);
        Flux<Employee> employeeFlux=Flux.fromIterable(employeeList);

        Mockito.when(repository.findAll()).thenReturn(employeeFlux);

        webTestClient.get()
                .uri("")
                .exchange()
                .expectStatus().isOk()
                .expectBodyList(Employee.class)
                .hasSize(2);

        Mockito.verify(repository,times(1)).findAll();
    }

    @Test
    void testUpdateEmployee(){
        Employee employee=new Employee();
        employee.setId(3);
        employee.setName("SYED");
        employee.setSalary(1800L);

        Mockito.when(repository.save(employee)).thenReturn(Mono.just(employee));

        webTestClient.put()
                .uri("/update")
                .contentType(MediaType.APPLICATION_JSON)
                .body(BodyInserters.fromValue(employee))
                .exchange()
                .expectStatus().isOk();

        Mockito.verify(repository,times(1)).save(employee);
    }

    @Test
    void testDeleteEmployee()
    {
        Mono<Void> voidReturn  = Mono.empty();
        Mockito
                .when(repository.deleteById(1))
                .thenReturn(voidReturn);

        webTestClient.delete().uri("/delete/{id}", 1)
                .exchange()
                .expectStatus().isOk();
    }
}
